/* MAKE-SUB-FILES:  Make files for submitting an entry from probability files. 
 *
 * Usage:  make-sub-files dataset-name method-prefix threshold
 *
 * This program must be run from inside a subdirectory of the "submissions"
 * directory.  It reads the files of predictive probabilities with names
 *
 *   ../../datasets/dataset-name/method-prefix.pr.train
 *   ../../datasets/dataset-name/method-prefix.pr.valid
 *   ../../datasets/dataset-name/method-prefix.pr.test
 * 
 * where "dataset-name" and "method-prefix" are the first two arguments.  It
 * uses these probabilities to make guesses, guessing "1" if the probability
 * of a "1" is greater than the specified threshold.  These guesses are 
 * written to the files
 *
 *   dataset-name_train.resu
 *   dataset-name_valid.resu
 *   dataset-name_test.resu
 *
 * The probability of the guess being correct is also computed, and these
 * probabilities are written to the files
 *
 *   dataset-name_train.conf
 *   dataset-name_valid.conf
 *   dataset-name_test.conf
 *
 * Finally, the balanced error rate on training and validation cases is
 * computed and printed, using the class labels in the files
 *
 *   ../../datasets/dataset-name/train.labels
 *   ../../datasets/dataset-name/valid.labels
 */

#include <stdio.h>

char *sets[] = { "train", "valid", "test", 0 };

main(int argc, char **argv)
{
  double threshold, pr;
  char pfile[100], rfile[100], cfile[100], lfile[100];
  char *dname, *mprefix;
  FILE *pf, *rf, *cf, *lf;
  int s, t, nn, np, nne, npe;
  char junk;

  if (argc!=4
   || sscanf(argv[3],"%lf%c",&threshold,&junk)!=1 
   || threshold<=0 || threshold>=1)
  { fprintf(stderr,
            "Usage: make-sub-files dataset-name method-prefix threshold\n");
    exit(1);
  }

  dname = argv[1];
  mprefix = argv[2];

  for (s = 0; sets[s]; s++)
  { 
    sprintf(pfile,"../../datasets/%s/%s.pr.%s",dname,mprefix,sets[s]);
    sprintf(rfile,"%s_%s.resu",dname,sets[s]);
    sprintf(cfile,"%s_%s.conf",dname,sets[s]);
    sprintf(lfile,"../../datasets/%s/%s.labels",dname,sets[s]);

    pf = fopen(pfile,"r");
    if (pf==NULL) 
    { fprintf(stderr,"Can't open %s - skipping\n",pfile);
      continue;
    }

    rf = fopen(rfile,"w");
    cf = fopen(cfile,"w");
    if (rf==NULL || cf==NULL)
    { fprintf(stderr,"Can't create results/confidence file - exitting\n");
      exit(1);
    }

    lf = fopen(lfile,"r");
    
    nn = np = nne = npe = 0;

    while (fscanf(pf,"%lf",&pr)==1)
    { if (pr<0 || pr>1)
      { fprintf(stderr,"Bad probability (%f) - exitting\n",pr);
        exit(1);
      }
      if (lf!=NULL) 
      { if (fscanf(lf,"%d",&t)!=1 || t!=1 && t!=-1)
        { fprintf(stderr,"Problem reading %s - exitting\n",lfile);
          exit(1);
        }
        if (t==1) np += 1;
        else nn += 1;
      }
      if (pr>threshold)
      { fprintf(rf,"1\n");
        fprintf(cf,"%f\n",pr);
        if (lf!=NULL && t==-1) nne += 1;
      }
      else
      { fprintf(rf,"-1\n");
        fprintf(cf,"%f\n",1-pr);
        if (lf!=NULL && t==1) npe += 1;
      }
    }

    if (lf!=NULL)
    { printf ("Balanced error rate for %s %s: %.4f\n",
               dname, sets[s], ((double)nne/nn + (double)npe/np)/2);
    }

    fclose(pf); 
    fclose(rf); 
    fclose(cf);
  }

  exit(0);
}
