# FIND PRINCIPLE COMPONENT VECTORS.  The data is in the first argument, with
# rows corresponding to cases, columns to features.  This procedure handles
# cases with fewer or more features than cases appropriately, returning as 
# many principle components as the minimum of the two.  The number of 
# principle components can be further reduced with the "k" argument.  The
# "center" and "scale" arguments (defaulting to FALSE) say whether the mean
# should be subtracted from each feature, and whether it should be divided
# by the standard deviation, before being used.

pca.vectors <- function (x, k=NULL, center=FALSE, scale=FALSE)
{ 
  shift <- 0
  mul <- 1

  if (center || scale)
  { x <- scale(x,center=center,scale=scale)
    if (center) shift <- -attr(x,"scaled:center")
    if (scale)  mul <- 1/attr(x,"scaled:scale")
  }

  if (nrow(x)<ncol(x))
  { m <- x %*% t(x)
    ev <- eigen(m,TRUE)
    if (is.null(k))
    { e <- t(x) %*% ev$vectors
    }
    else
    { e <- t(x) %*% ev$vectors[,1:k]
    }
  }
  else
  { m <- t(x) %*% x
    ev <- eigen(m,T)
    e <- ev$vectors
  }

  v <- ev$values / nrow(x)

  if (!is.null(k))
  { e <- e[,1:k]
    v <- v[1:k]
  }

  for (i in 1:ncol(e))
  { e[,i] <- e[,i] / sqrt(sum(e[,i]^2))
  }

  return (e, v, shift, mul)
}


# FIND PROJECTIONS OF DATA ONTO FIRST K PRINCIPLE COMPONENTS.  The first
# argument is the list of principle component vectors, shift, and scale
# factors returned by pca.vectors.  The second argument is the data to
# project, with rows being cases and columns feature values.  If "normal" 
# is TRUE (not the default), the result includes a final value for the
# amount normal to the space of the k principle component vectors.

pca.proj <- function (pc, x, k=ncol(vectors$e), normal=FALSE)
{
  if (!all(pc$shift==0) || !all(pc$mul==1))
  { shift <- rep(pc$shift,length.out=ncol(x))
    scale <- rep(pc$mul,length.out=ncol(x))
    for (i in 1:ncol(x))
    { x[,i] <- (x[,i]+shift[i])*scale[i]
    }
  }

  if (k==ncol(pc$e))
  { nx <- x %*% pc$e
  }
  else
  { nx <- x %*% pc$e[,1:k]
  }

  if (normal)
  { d2 <- apply(x^2,1,sum)
    nd2 <- apply(nx^2,1,sum)
    nx <- cbind (nx, sqrt(pmax(d2-nd2,0)))
  }

  nx
}


# WRITE PROJECTIONS OF DATA ONTO PRINCIPLE COMPONENTS TO FILES.  Arguments
# are the principle component vectors, as returned by pca.vectors, the
# name of the file, and the data matrices, defaulting to the standard
# training, validation, and test matrices.  If "normal" is TRUE (not the
# default), includes a final variable for the amount normal to the space of 
# these principle components.

write.pca <- function (pc, f, trn.x=train.x, val.x=valid.x, tst.x=test.x,
                       normal=FALSE)
{ 
  k=ncol(pc$e)

  write (t(pca.proj(pc,trn.x,k,normal=normal)), paste("train.data.",f,sep=""), 
         ncolumns=k+normal)
  write (t(pca.proj(pc,val.x,k,normal=normal)), paste("valid.data.",f,sep=""), 
         ncolumns=k+normal)
  write (t(pca.proj(pc,tst.x,k,normal=normal)), paste("test.data.",f,sep=""), 
         ncolumns=k+normal)
}
